#!/usr/bin/perl -w
#use strict;

# Allin Cottrell, May 1999.
#
# convert regular quarterly representation to Y/M/D
# and output an appropriate xtics line for gnuplot
#
# The input datafile should have quarterly dates in the form
# "YY{YY}.Q" (e.g. 1956.1, 1999.4, 60.3) in the first data
# column.  Any comment lines should begin with "#".

my (@fields, @datebits); 
my ($datafile, $year, $quart, $qlong, $i, $j, $nf, $nl);
my ($every_mark, $start_year, $maxtics, $maxmtics);
$maxmtics = 80;  # max. num. of unlabeled tics to show

if ($#ARGV > -1) { $datafile = $ARGV[0]; }
else { die "gp_quarters.pl: no datafile specified.\n"; }
open (INP, "$datafile") || 
   die "couldn't open $datafile for reading.\n";
open (DAT, ">./qdata.tmp") && open (TIC, ">./tictmp.gp") || 
   die "couldn't open temporary files for writing.\n";
if ($#ARGV > 0) { $maxtics = $ARGV[1]; }
else { $maxtics = 12; } 
 
# output boiler-plate for gnuplot
print TIC "set xdata time\n";
print TIC "set timefmt '%Y/%m/%d'\n";
print TIC "set xtics nomirror\n";
print TIC "set xtics (\\\n";

# initialize dummy data file
print DAT "# modified data for gnuplot\n";

while (<INP>) { (/^#/) || $nl++; } # count data lines
close (INP);
# calculate frequency of major tics, based on $maxtics
$every_mark = 4 * int( .5 + $nl / ( 4 * $maxtics ) );
if ($every_mark == 0) { $every_mark = 4; }

$j = 0; $start_year = 0;
open (INP, "$datafile");
while (<INP>) {
   if (/^#/) { next; } # comment line -- ignore
   $j++; 
   @fields = split; $nf = $#fields;
   @datebits = split(/\./,$fields[0]);
   ($#datebits > 0) || die 
      "First column of $datafile does not seem to be quarterly dates.\n"; 
   $year = $datebits[0]; $quart = $datebits[1];
   if ( $quart eq "1" ) { 
      $qlong = "01/01";
      if ( $start_year == 0 ) { $start_year = $j; }
   }
   elsif ( $quart eq "2" ) { $qlong = "04/01"; }
   elsif ( $quart eq "3" ) { $qlong = "07/01"; }
   elsif ( $quart eq "4" ) { $qlong = "10/01"; }
   else { 
      die "$datafile contains an invalid quarter designator.\n"; 
   }
   print DAT "$year/$qlong";
   for ($i = 1; $i <= $nf; $i++) { print DAT " $fields[$i]"; }
   print DAT "\n";
   if ( $nl <= $maxtics ) {  # few points: mark all tics
         print TIC "'$year.$quart' '$year/$qlong',\\\n"; 
   } else {            # many points: just mark $maxtics of them
      if ( $j % $every_mark == $start_year ) {
         print TIC "'$year.$quart' '$year/$qlong',\\\n";
      } else {         # not a labeled tic
         if ( $nl < $maxmtics ) 
	   { print TIC "\"\" '$year/$qlong',\\\n" ; }
      }
   }
}
print TIC "'' '')\n"; 
close (DAT); close (TIC); close (INP); 
exit;





